@@ -31,6 +31,8 @@ import torchvision.models
 import re
 from nemo.transf.common import *
 
+BN_PRECISION_MAX = 20
+
 def _absorb_affine_bn(self):
     for n,m in self.named_modules():
         if m.__class__.__name__ == "BatchNorm2d":
@@ -45,7 +47,7 @@ def _prime_affine_bn(self):
             m.weight.data[:] = m.weight.data[:] * torch.sqrt(m.running_var.data[:] + m.eps)
             m.bias.data[:] = m.bias.data[:] + m.weight.data[:] * m.running_mean.data[:] / torch.sqrt(m.running_var.data[:] + m.eps)
 
-def _freeze_bn(self, reset_stats=False):
+def _freeze_bn(self, reset_stats=False, disable_grad=False):
     r"""Sets :py:class:torch.nn.BatchNorm2d` layers to not collect statistics and keep the current `running_var` and `running_mean`.
     
     """
@@ -69,9 +71,61 @@ def _freeze_bn(self, reset_stats=False):
                 m.running_mean[:] = 0.
             m.track_running_stats = True
             m.eval()
+            if disable_grad:
+                m.weight.requires_grad = False
+                m.bias.requires_grad = False
+
+def _calibrate_bn_pact(self, calib_dict={}, kappa_bit_default=16, lamda_bit_default=24, kappa_dict=None, lamda_dict=None, range_factor=8, minmax=False, **kwargs):
+    r"""Calibrates BN layer quantization for :py:class:`nemo.quant.pact.PACT_QuantizedBatchNormNd` layers.
+    Using BN min-max statistics previously calculated, this method calibrates the number of bits used in BN parameters
+    so that both the BN output and the `kappa`, `lamda` affine parameters are representable.
+    
+    :param kappa_bit_default: Default maximum number of bits for BN multiplicative parameter (can be overridden by `kappa_dict`); default 16.
+    :type  kappa_bit_default: int
+    :param lamda_bit_default: Default maximum number of bits for BN additive parameter (can be overridden by `lamda_dict`); default 32.
+    :type  lamda_bit_default: int
+    :param kappa_dict: dictionary of maximum number of bits for `kappa` in specific layers; overrides `kappa_bit_default`. Default None.
+    :type  kappa_dict: `dict` or `collections.OrderedDict`
+    :param lamda_dict: dictionary of maximum number of bits for `lamda` in specific layers; overrides `lamda_bit_default`. Default None.
+    :type  lamda_dict: `dict` or `collections.OrderedDict`
+    
+    """
+
+    if not calib_dict:
+        calib_dict = get_calib_dict_from_supernodes(self)
+
+    module_dict = {}
+    for n,m in self.named_modules():
+        if (m.__class__.__name__ == "PACT_Act" or \
+            m.__class__.__name__ == "PACT_QuantizedBatchNormNd"):
+            module_dict[n] = m
+
+    for n,m in self.named_modules():
+        if m.__class__.__name__ == "PACT_QuantizedBatchNormNd":
+            kappa_max = m.kappa.abs().max()
+            lamda_max = m.lamda.abs().max()
+            if minmax:
+                out_range = max(m.max.abs(), m.min.abs())
+            else: # if we do not have minmax statistics, approximate with alpha mult by a range_factor
+                out_range = module_dict[calib_dict[n]].alpha * range_factor
+            if kappa_dict is not None:
+                kappa_bit_lim = kappa_dict[n]
+            else:
+                kappa_bit_lim = kappa_bit_default
+            if lamda_dict is not None:
+                lamda_bit_lim = lamda_dict[n]
+            else:
+                lamda_bit_lim = lamda_bit_default
+            eps_lim = max(out_range, lamda_max) / (2**(lamda_bit_lim - 1) - 1)
+            eps_kappa_lim = eps_lim / m.eps_in
+            kappa_bits = min(int(min(torch.log2(1 + 2*kappa_max / eps_kappa_lim).floor(), kappa_bit_lim)), BN_PRECISION_MAX)
+            lamda_bits = min(lamda_bit_lim, BN_PRECISION_MAX)
+            m.precision_kappa.set_bits(kappa_bits)
+            m.precision_lamda.set_bits(lamda_bits)
+            # print("%s %d %d" % (n, kappa_bits, lamda_bits))
 
 def _unfreeze_bn(self):
-    r"""Sets :py:class:torch.nn.BatchNorm2d` layers to collect statistics and update `running_var` and `running_mean`.
+    r"""Sets :py:class:`torch.nn.BatchNorm2d` layers to collect statistics and update `running_var` and `running_mean`.
     
     """
 
@@ -79,6 +133,38 @@ def _unfreeze_bn(self):
         if m.__class__.__name__ == "BatchNorm2d":
             m.train()
 
+def _prune_empty_bn_pact(self, bn_dict={}, threshold=None):
+    if not bn_dict:
+        bn_dict = get_bn_dict_from_supernodes(self)
+
+    module_dict = {}
+    for n,m in self.named_modules():
+        if (m.__class__.__name__ == "PACT_Conv2d" or \
+            m.__class__.__name__ == "PACT_Conv1d" or \
+            m.__class__.__name__ == "PACT_Linear" or \
+            m.__class__.__name__ == "BatchNorm2d" or \
+            m.__class__.__name__ == "BatchNorm1d" or \
+            m.__class__.__name__ == "PACT_QuantizedBatchNormNd"):
+            module_dict[n] = m
+    for n_before in bn_dict.keys():
+        n_after  = bn_dict[n_before]
+        m_before = module_dict[n_before]
+        m_after  = module_dict[n_after]
+        if threshold is None:
+            try:
+                eps = (m_before.W_alpha + m_before.W_beta) / (2.**m_before.W_precision.get_bits()-1)
+            except AttributeError:
+                continue
+        else:
+            eps = threshold
+        if m_before.bias is not None:
+            continue
+        try:
+            m_after.kappa.data[m_before.weight.data.flatten(1).abs().max(1)[0] < eps] = 0.
+        except AttributeError:
+            m_after.weight.data[m_before.weight.data.flatten(1).abs().max(1)[0] < eps] = 0.
+            m_after.running_var.data[m_before.weight.data.flatten(1).abs().max(1)[0] < eps] = 1.
+
 def _fold_bn_pact(self, bn_dict={}, bn_inv_dict={}, eps=None, phi_inv=0., reset_alpha=True):
     r"""Performs batch-normalization folding following the algorithm presented in
     https://arxiv.org/abs/1905.04166. It performs both normal folding and inverse
@@ -109,8 +195,7 @@ def _fold_bn_pact(self, bn_dict={}, bn_inv_dict={}, eps=None, phi_inv=0., reset_
             m.__class__.__name__ == "PACT_Linear" or \
             m.__class__.__name__ == "BatchNorm2d" or \
             m.__class__.__name__ == "BatchNorm1d" or \
-            m.__class__.__name__ == "PACT_QuantizedBatchNorm2d" or \
-            m.__class__.__name__ == "PACT_QuantizedBatchNorm1d" ):
+            m.__class__.__name__ == "PACT_QuantizedBatchNormNd"):
             module_dict[n] = m
     param = {}
     bn_list = list(bn_dict.values()) + list(bn_inv_dict.keys())
@@ -123,8 +208,7 @@ def _fold_bn_pact(self, bn_dict={}, bn_inv_dict={}, eps=None, phi_inv=0., reset_
                 eps = m.eps
             except AttributeError:
                 eps = 0.
-        if m.__class__.__name__ == "PACT_QuantizedBatchNorm2d" or \
-           m.__class__.__name__ == "PACT_QuantizedBatchNorm1d":
+        if m.__class__.__name__ == "PACT_QuantizedBatchNormNd":
             gamma = m.kappa.data[:].clone().detach().cpu().flatten()
             beta  = m.lamda.data[:].clone().detach().cpu().flatten()
             sigma = torch.ones_like(gamma).flatten()
@@ -189,8 +273,7 @@ def _fold_bn_pact(self, bn_dict={}, bn_inv_dict={}, eps=None, phi_inv=0., reset_
     # neutralize BatchNorm's
     for n in bn_list:
         m = module_dict[n]
-        if m.__class__.__name__ == "PACT_QuantizedBatchNorm2d" or \
-           m.__class__.__name__ == "PACT_QuantizedBatchNorm1d":
+        if m.__class__.__name__ == "PACT_QuantizedBatchNormNd":
             m.kappa.data[:] = 1.
             m.lamda.data[:] = 0.
         else:
