@@ -30,7 +30,7 @@ import math
 import torchvision.models
 import re
 
-__all__ = [ "reshape_before", "reshape_after", "weight_range", "onnx_name_2_pytorch_name", "get_bn_dict_from_supernodes", "get_equalize_dict_from_supernodes" ]
+__all__ = [ "reshape_before", "reshape_after", "weight_range", "weight_min", "weight_max", "onnx_name_2_pytorch_name", "get_bn_dict_from_supernodes", "get_equalize_dict_from_supernodes", "get_calib_dict_from_supernodes" ]
 
 def reshape_before(m, s):
     if   m.__class__.__name__ == "PACT_Conv2d":
@@ -56,6 +56,30 @@ def reshape_after(m, s):
     else:
         return s
 
+def weight_max(m, range_idx):
+    if   m.__class__.__name__ == "PACT_Conv2d":
+        if m.groups == m.weight.shape[0]:
+            range_idx = 0 # for DW-conv, always marginalize idx 1
+        return m.weight.max(3)[0].max(2)[0].max(1-range_idx)[0] 
+    elif m.__class__.__name__ == "PACT_Conv1d":
+        return m.weight.max(2)[0].max(1-range_idx)[0]
+    elif m.__class__.__name__ == "PACT_Linear":
+        return m.weight.max(1-range_idx)[0]
+    elif m.__class__.__name__ == "BatchNorm1d" or m.__class__.__name__ == "BatchNorm2d":
+        return m.weight.data.abs()
+
+def weight_min(m, range_idx):
+    if   m.__class__.__name__ == "PACT_Conv2d":
+        if m.groups == m.weight.shape[0]:
+            range_idx = 0 # for DW-conv, always marginalize idx 1
+        return m.weight.min(3)[0].min(2)[0].min(1-range_idx)[0]
+    elif m.__class__.__name__ == "PACT_Conv1d":
+        return m.weight.min(2)[0].min(1-range_idx)[0]
+    elif m.__class__.__name__ == "PACT_Linear":
+        return m.weight.min(1-range_idx)[0]
+    elif m.__class__.__name__ == "BatchNorm1d" or m.__class__.__name__ == "BatchNorm2d":
+        return m.weight.data.abs()
+
 def weight_range(m, range_idx, symmetric=False):
     if   m.__class__.__name__ == "PACT_Conv2d":
         if m.groups == m.weight.shape[0]:
@@ -83,8 +107,8 @@ def onnx_name_2_pytorch_name(name):
     return '.'.join(name_parts)
  
 def get_equalize_dict_from_supernodes(net):
-    eq_dict = {}
-    act_dict = {}
+    eq_dict = OrderedDict()
+    act_dict = OrderedDict()
     # check all supernodes for ACT and CONV layers
     lin = {}
     act = {}
@@ -105,7 +129,7 @@ def get_equalize_dict_from_supernodes(net):
     return eq_dict, act_dict
 
 def get_bn_dict_from_supernodes(net):
-    bn_dict = {}
+    bn_dict = OrderedDict()
     # check all supernodes for BN and CONV layers
     for k,ssn in net.graph.get_supernodes().items():
         bn = []
@@ -115,14 +139,14 @@ def get_bn_dict_from_supernodes(net):
         for n in sn:
             if isinstance(n[1], torch.nn.BatchNorm2d) or \
                isinstance(n[1], torch.nn.BatchNorm1d) or \
-               isinstance(n[1], PACT_QuantizedBatchNorm2d):
+               isinstance(n[1], PACT_QuantizedBatchNormNd):
                 bn.append(n[0])
             if isinstance(n[1], PACT_Conv2d) or \
                isinstance(n[1], PACT_Conv1d) or \
                isinstance(n[1], PACT_Linear):
                 lin.append(n[0])
         if len(lin) > 1 or len(bn) > 1:
-            print("[Error] Supernode analysis identified multiple BN or LIN layers when tring to fold! Aborting folding...")
+            print("[Error] Supernode analysis identified multiple BN or LIN layers in supernode")
             print(lin, bn)
             return
         try:
@@ -131,3 +155,25 @@ def get_bn_dict_from_supernodes(net):
             pass
     return bn_dict
 
+def get_calib_dict_from_supernodes(net):
+    calib_dict = OrderedDict()
+    # check all supernodes for BN and ACT layers
+    for k,ssn in net.graph.get_supernodes().items():
+        bn = []
+        sn = ssn['supernode']
+        prev = ssn['previous']
+        for n in sn:
+            if isinstance(n[1], torch.nn.BatchNorm2d) or \
+               isinstance(n[1], torch.nn.BatchNorm1d) or \
+               isinstance(n[1], PACT_QuantizedBatchNormNd):
+                bn.append(n[0])
+        if len(bn) > 1:
+            print("[Error] Supernode analysis identified multiple BN layers in supernode")
+            print(bn)
+            return
+        try:
+            calib_dict[bn[0]] = k
+        except IndexError:
+            pass
+    return calib_dict
+
