@@ -142,7 +142,7 @@ def _get_clip_parameters_pact(self):
         if name[-5:] == 'alpha':
             yield param
 
-def _reset_alpha_weights_pact(self, **kwargs):
+def _reset_alpha_weights_pact(self, method='standard', **kwargs):
     r"""Resets parameter `W_alpha`.
     
     """
@@ -153,20 +153,3 @@ def _reset_alpha_weights_pact(self, **kwargs):
             m.__class__.__name__ == "PACT_Linear"):
             m.reset_alpha_weights(**kwargs)
 
-def _qd_stage(self, eps_in=None, add_input_bias_dict={}, remove_bias_dict={}, **kwargs):
-    self.harden_weights()
-    if add_input_bias_dict:
-        self.add_input_bias(add_input_bias_dict)
-    if remove_bias_dict:
-        self.remove_bias(remove_bias_dict)
-    nemo.transform.bn_quantizer(self, **kwargs)
-    self.set_deployment(eps_in=eps_in)
-
-def _id_stage(self, eps_in=None, **kwargs):
-    if self.stage == 'fq':
-        self.qd_stage(eps_in=eps_in, **kwargs)
-    self.stage = 'id'
-    if eps_in is None:
-        eps_in = self.eps_in
-    nemo.transform.integerize_pact(self, eps_in=eps_in)
-
