@@ -41,7 +41,7 @@ from nemo.transf.pruning import *
 from nemo.transf.statistics import *
 from nemo.transf.utils import *
 
-def quantize_pact(module, W_bits=4, x_bits=4, dummy_input=None, remove_dropout=False):
+def quantize_pact(module, W_bits=4, x_bits=4, dummy_input=None, remove_dropout=False, **kwargs):
     r"""Takes a PyTorch module and makes it quantization-aware with PACT, recursively.
 
     The function follows recursively the data structures containing PyTorch layers (typically as hierarchical lists, e.g.
@@ -95,7 +95,7 @@ def quantize_pact(module, W_bits=4, x_bits=4, dummy_input=None, remove_dropout=F
     else:
         module.graph = None
     module.stage = 'fq'
-    module = _hier_quantizer_pact(module, module.graph)
+    module = _hier_quantizer_pact(module, module.graph, **kwargs)
     if hasattr(module, 'graph'):
         if module.graph is not None:
             module.graph.rebuild_module_dict()
@@ -109,10 +109,15 @@ def quantize_pact(module, W_bits=4, x_bits=4, dummy_input=None, remove_dropout=F
     module.fold_bn                     = types.MethodType(nemo.transf.bn._fold_bn_pact, module)
     module.fold_bn_withinv             = types.MethodType(nemo.transf.bn._fold_bn_pact, module)
     module.fold_thresholds             = types.MethodType(nemo.transf.bn._threshold_folding_pact, module)
+    module.prune_empty_bn              = types.MethodType(nemo.transf.bn._prune_empty_bn_pact, module)
+    module.calibrate_bn                = types.MethodType(nemo.transf.bn._calibrate_bn_pact, module)
     module.get_eps_at                  = types.MethodType(nemo.transf.deploy._get_eps_at, module)
     module.set_eps_in                  = types.MethodType(nemo.transf.deploy._set_eps_in_pact, module)
+    module.round_weights               = types.MethodType(nemo.transf.deploy._round_weights_pact, module)
     module.harden_weights              = types.MethodType(nemo.transf.deploy._harden_weights_pact, module)
     module.set_deployment              = types.MethodType(nemo.transf.deploy._set_deployment_pact, module)
+    module.qd_stage                    = types.MethodType(nemo.transf.deploy._qd_stage, module)
+    module.id_stage                    = types.MethodType(nemo.transf.deploy._id_stage, module)
     module.export_precision            = types.MethodType(nemo.transf.export._export_precision, module)
     module.export_weights_legacy_int16 = types.MethodType(nemo.transf.export._export_weights_legacy_int16, module)
     module.change_precision            = types.MethodType(nemo.transf.utils._change_precision_pact, module)
@@ -122,15 +127,15 @@ def quantize_pact(module, W_bits=4, x_bits=4, dummy_input=None, remove_dropout=F
     module.get_nonclip_parameters      = types.MethodType(nemo.transf.utils._get_nonclip_parameters_pact, module)
     module.set_train_loop              = types.MethodType(nemo.transf.utils._set_train_loop_pact, module)
     module.unset_train_loop            = types.MethodType(nemo.transf.utils._unset_train_loop_pact, module)
-    module.qd_stage                    = types.MethodType(nemo.transf.utils._qd_stage, module)
-    module.id_stage                    = types.MethodType(nemo.transf.utils._id_stage, module)
     module.prune_weights               = types.MethodType(nemo.transf.pruning._prune_weights_pact, module)
     module.equalize_weights_dfq        = types.MethodType(nemo.transf.equalize._equalize_weights_dfq_pact, module)
+    module.equalize_weights_lsq        = types.MethodType(nemo.transf.equalize._equalize_weights_lsq_pact, module)
     module.equalize_weights_unfolding  = types.MethodType(nemo.transf.equalize._equalize_weights_unfolding_pact, module)
     module.statistics_act              = types.MethodType(nemo.transf.statistics._statistics_act_pact, module)
     module.set_statistics_act          = types.MethodType(nemo.transf.statistics._set_statistics_act_pact, module)
     module.get_statistics_act          = types.MethodType(nemo.transf.statistics._get_statistics_act_pact, module)
     module.unset_statistics_act        = types.MethodType(nemo.transf.statistics._unset_statistics_act_pact, module)
+    module.statistics_bn               = types.MethodType(nemo.transf.statistics._statistics_bn_pact, module)
     module.set_statistics_bn           = types.MethodType(nemo.transf.statistics._set_statistics_bn_pact, module)
     module.unset_statistics_bn         = types.MethodType(nemo.transf.statistics._unset_statistics_bn_pact, module)
     module.W_precision = Precision(W_bits, None)
@@ -172,7 +177,7 @@ def _hier_bn_quantizer(module, **kwargs):
         sigma = torch.sqrt(module.running_var.data[:] + module.eps).clone().detach()
         mu = module.running_mean.data[:].clone().detach()
         dimensions = 1 if module.__class__.__name__ == 'BatchNorm1d' else 2
-        module = PACT_QuantizedBatchNorm2d(kappa=gamma/sigma, lamda=beta-gamma/sigma*mu, dimensions=dimensions, **kwargs)
+        module = PACT_QuantizedBatchNormNd(kappa=gamma/sigma, lamda=beta-gamma/sigma*mu, dimensions=dimensions, **kwargs)
         return module
     else:
         for n,m in module.named_children():
@@ -180,8 +185,7 @@ def _hier_bn_quantizer(module, **kwargs):
         return module
 
 def _hier_bn_dequantizer(module):
-    if module.__class__.__name__ == 'PACT_QuantizedBatchNorm2d':# or \
-    #    module.__class__.__name__ == 'BatchNorm1d':
+    if module.__class__.__name__ == 'PACT_QuantizedBatchNormNd':
         gamma = module.kappa.data[:].clone().detach().flatten()
         beta = module.lamda.data[:].clone().detach().flatten()
         module = torch.nn.BatchNorm2d(weight=gamma, bias=beta)
@@ -197,17 +201,17 @@ def _hier_integerizer(module, **kwargs):
         module.__class__.__name__ == "PACT_Linear"):
         module.integerize_weights(**kwargs)
         return module
-    elif (module.__class__.__name__ == "PACT_QuantizedBatchNorm2d"):
-        module = PACT_IntegerBatchNorm2d(kappa=module.kappa, lamda=module.lamda, eps_in=module.eps_in, eps_kappa=module.eps_kappa, eps_lamda=module.eps_lamda)
+    elif (module.__class__.__name__ == "PACT_QuantizedBatchNormNd"):
+        module = PACT_IntegerBatchNormNd(kappa=module.kappa, lamda=module.lamda, eps_in=module.eps_in, eps_kappa=module.eps_kappa, eps_lamda=module.eps_lamda)
         module.integerize_weights(**kwargs)
     elif (module.__class__.__name__ == "PACT_Act"):
-        module = PACT_IntegerAct(precision=module.precision, eps_in=module.eps_in, alpha=module.alpha)
-        module.set_output_eps()
+        module = PACT_IntegerAct(precision=module.precision, eps_in=module.eps_in, eps_out=module.eps_static, alpha=module.alpha_static, **kwargs)
+        module.set_output_eps(**kwargs)
     elif (module.__class__.__name__ == "PACT_IntegerAdd"):
         module.integerized = True
     elif (module.__class__.__name__ == "AvgPool2d"):
         module = PACT_IntegerAvgPool2d(module.kernel_size, stride=module.stride, padding=module.padding, ceil_mode=module.ceil_mode,
-            count_include_pad=module.count_include_pad, requantization_factor=16)
+            count_include_pad=module.count_include_pad)
     else:
         for n,m in module.named_children():
             module._modules[n] = _hier_integerizer(m, **kwargs)
@@ -222,7 +226,7 @@ def _hier_thresholdizer_pact(module):
             module._modules[n] = _hier_thresholdizer_pact(m)
         return module
 
-def _hier_quantizer_pact(module, graph=None):
+def _hier_quantizer_pact(module, graph=None, **kwargs):
     if module.__class__.__name__ == 'Conv2d':
         W = module.weight.data
         try:
@@ -279,17 +283,17 @@ def _hier_quantizer_pact(module, graph=None):
             module.bias.data = b.clone()
         return module
     elif module.__class__.__name__ == 'ReLU6':
-        module = PACT_Act(alpha=6.)
+        module = PACT_Act(alpha=6., **kwargs)
         return module
     elif module.__class__.__name__ == 'ReLU':
-        module = PACT_Act()
+        module = PACT_Act(**kwargs)
         return module
     elif module.__class__.__name__ == 'LeakyReLU':
-        module = PACT_Act(leaky=module.negative_slope)
+        module = PACT_Act(leaky=module.negative_slope, **kwargs)
         return module
     else:
         for n,m in module.named_children():
-            module._modules[n] = _hier_quantizer_pact(m)
+            module._modules[n] = _hier_quantizer_pact(m, **kwargs)
         return module
 
 def _hier_dequantizer_pact(module):
